import { join } from '@waiting/shared-core';
import { initialMsgPrefixOpts, initialSpawnOpts } from './config';
export function processOpts(options) {
    const { args, initialRxRunOpts } = options;
    const spawnOpts = processSpawnOpts(initialRxRunOpts, options.spawnOpts);
    const cmd = processCommand(options.command, initialRxRunOpts.msgPrefixOpts.errPrefix, spawnOpts);
    // eslint-disable-next-line @typescript-eslint/prefer-optional-chain
    const runArgs = args && args.length ? [...args] : [];
    const errScript = runArgs.length
        ? `"${cmd} ${runArgs.join(' ')}"`
        : `"${cmd}"`;
    return {
        command: cmd,
        args: runArgs,
        spawnOpts,
        errScript,
    };
}
function processSpawnOpts(defaultRxrunOpts, options) {
    const opts = (options ? { ...initialSpawnOpts, ...options } : initialSpawnOpts);
    opts.shell = true;
    /* istanbul ignore next */
    if (process.platform === 'win32') {
        opts.windowsVerbatimArguments = true;
    }
    /* istanbul ignore else */
    if (typeof opts.stderrMaxBufferSize !== 'number' || opts.stderrMaxBufferSize < 0) {
        opts.stderrMaxBufferSize = +defaultRxrunOpts.stderrMaxBufferSize;
    }
    opts.msgPrefixOpts = { ...initialMsgPrefixOpts };
    return opts;
}
function processCommand(command, errPrefix, spawnOpts) {
    let cmd = command ? command.trim() : '';
    /* istanbul ignore else */
    if (!cmd) {
        throw new TypeError(`${errPrefix}\nRun command is blank`);
    }
    const cmdLead = cmd.slice(0, 3).replace(/\\/ug, '/');
    /* istanbul ignore else */
    if (spawnOpts.cwd && (cmdLead === '../' || cmdLead.startsWith('./'))) {
        cmd = join(spawnOpts.cwd, cmd);
    }
    /* istanbul ignore else */
    if (typeof spawnOpts.maxCmdLength === 'number'
        && spawnOpts.maxCmdLength > 0
        && cmd.length > spawnOpts.maxCmdLength) {
        throw new TypeError(`${errPrefix}\nCommand length exceed ${spawnOpts.maxCmdLength}\n` + cmd.slice(1024) + ' ...');
    }
    return cmd;
}
